@@ -15,7 +15,7 @@ our @EXPORT_OK = qw{
     test_stack_hide test_stack_shift test_stack_stop test_stack_no_start
     test_stack_alter test_stack_shift_and_hide test_stack_shift_short
     test_stack_hide_short test_stack_shift_and_alter test_stack_full_combo
-    test_stack_restart
+    test_stack_restart test_stack_special
 };
 
 sub NA() { \&NA }
@@ -376,6 +376,24 @@ sub restart_4 { my $code = shift; @_ = (@_); mask_frame('stop' => 1); restart_5(
 sub restart_5 { my $code = shift; @_ = (@_); restart_6($code, 'f') }                                # line 11
 sub restart_6 { my $code = shift; @_ = (@_); mask_call({no_start => 1}, $code) }                    # line 12
 
+
+
+
+#line 1 "mask_test_special.pl"
+sub test_stack_special {                 # line 1
+    my ($callback) = @_;                 # line 2
+    mask_frame(stop => 1, hide => 1);    # line 3
+    special_1($callback, 'a');           # line 4
+}                                        # line 5
+
+sub special_1 { my $code = shift; @_ = (@_); special_2($code, 'b') }
+sub special_2 { my $code = shift; @_ = (@_); unimport($code, 'c') }
+sub unimport {  my $code = shift; @_ = (@_); mask_frame(hide => 1); special_3($code, 'd') }
+sub special_3 { my $code = shift; @_ = (@_); special_4($code, 'e') }
+sub special_4 { my $code = shift; @_ = (@_); special_5($code, 'f') }
+sub special_5 { my $code = shift; @_ = (@_); mask_frame(stop => 1); special_6($code, 'g') }
+sub special_6 { my $code = shift; @_ = (@_); $code->() }
+
 1;
 
 =pod
